package ec.simple;

import java.io.File;
import java.io.IOException;
import java.text.DateFormat;
import java.text.SimpleDateFormat;
import java.time.Duration;
import java.util.Date;

import ec.Launcher;
import ec.EvolutionState;
import ec.Statistics;
import ec.util.Parameter;


/**
 * Captures and saves timing and seed statistics 
 * Assumes execution from ec.Launcher.main(...);
 * @author      Michael Gircys <mg12vp@brocku.ca>
 * @version     23.1.0
 * @since       23.1.0
 */
@SuppressWarnings("serial")
public class SimpleSeedTimeStatistics extends Statistics 
{

	public static final String P_STATISTICS_FILE = "file";
	public int statisticslog = 0;  // stdout

	private Date dateStart = null;
	private Date dateEnd   = null;
	private static final DateFormat dateFormat = new SimpleDateFormat("yyyy-MM-dd HH:mm:ss");


	@Override
	public void setup(final EvolutionState state, final Parameter base)
	{
		super.setup(state,base);
		
		// Output Setup 
		File statisticsFile = state.parameters.getFile(
			base.push(P_STATISTICS_FILE),null);
		if( statisticsFile != null )
			try { 
				statisticslog = state.output.addLog(statisticsFile, false, false); 
			} catch (IOException i){
				state.output.fatal("An IOException occurred while trying to create the log " + statisticsFile + ":\n" + i);
			}
		
		// Start Time
		dateStart = new Date();
		state.output.println( ("Launched at  : " + dateFormat.format(dateStart)), statisticslog );
		
		// Seeds
		int[] seeds = Launcher.GetSeeds();
		for(int i = 0; i < state.random.length; i++)
			state.output.println( ("Seed " + i + " State : " + seeds[i]), statisticslog );
		
	}

	@Override
	public void finalStatistics(final EvolutionState state, final int result)
	{
		super.finalStatistics(state, result);
		
		// End Time
		dateEnd = new Date();
		state.output.println( ("Completed at : " + dateFormat.format(dateEnd)), statisticslog );
		
		// Elapsed Time
		Duration duration = Duration.between(dateStart.toInstant(), dateEnd.toInstant());
		state.output.println( ("Elapsed time : " + duration.toString()), statisticslog );
	}

}
